#include "adventskalender.h"

#include <Qt>
#include <QtGui>
#include <QtGui/QPainter>
#include <QtGui/QColor>
#include <QtCore>
#include <QtDebug>

/* Use: instanciate it, fill it with gifts and give it away as binary.

#include <QtGui/QApplication>
#include "adventskalender.h"

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    AdventsKalender w;
    w.addGiftForDay(1," .. some bucks");
    w.addGiftForDay(2," .. some bees");
    w.addGiftForDay(3," .. some berries");
    // and so on till
    w.addGiftForDay(24," .. some coffee");
    w.show();
    return a.exec();
}
*/

AdventsKalender::AdventsKalender(QWidget *parent, Qt::WindowFlags f)
    : QWidget(parent,f)
{
    setWindowTitle(tr("Christmas Calender"));

    QGridLayout * lay = new QGridLayout(this);

    QHBoxLayout * hbl = 0;

    m_mapper = new QSignalMapper(this);
    connect(m_mapper, SIGNAL(mapped(int)), this, SLOT(showGiftForDay(int)));

    for(quint8 i=0; i<24; ++i)
    {
        const QString day(QString::number(i+1));

        QToolButton * tb = new QToolButton(this);
        tb->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        tb->setText(day+".");


        // icons bzw text wenn kein Icon da
        {
            QImage pix(":icons/day"+day);
            if (! pix.isNull())
            {
                QIcon icon(QPixmap::fromImage(pix));

                tb->setIcon(icon);
                tb->setIconSize(QSize(48,48));
                tb->setFont(QFont("Arial",8,QFont::Bold));
            }
            else
            {
                tb->setFont(QFont("Arial",24,QFont::Bold));
            }
        }

        // mapping zwischen button und slot via signal mapper
        {
            connect(tb, SIGNAL(clicked()), m_mapper, SLOT(map()));
            m_mapper->setMapping(tb,i+1); // int signal
        }

        // tannenbaum layout
        {
            switch (i)
            {
                // machn neues layout mit stretch am anfang
            case 0:
                hbl = new QHBoxLayout;
                hbl->addStretch(1);
                break;
                // mach nen stretch ans ende und fügs ein, machn neues layout mit stretch am anfang
            case 1:
            case 3:
            case 6:
            case 10:
            case 15:
            case 21:
            case 22:
                hbl->addStretch(1);
                lay->addLayout(hbl,lay->rowCount(),0);
                hbl = new QHBoxLayout;
                hbl->addStretch(1);
                break;
            }
            hbl->addWidget(tb);

            // mach nen stretch ans ende und fügs ein
            if (i==23) {
                hbl->addStretch(1);
                lay->addLayout(hbl,lay->rowCount(),0);
                hbl = 0;
            }
        }
    }

    // add about
    {
        hbl = new QHBoxLayout;
        hbl->addStretch(1);

        QVBoxLayout * vbl = new QVBoxLayout;
        vbl->addStretch(1);
        hbl->addLayout(vbl);

        QToolButton * tb = new QToolButton(this);
        connect(tb,SIGNAL(clicked()),this,SLOT(aboutThis()));

        tb->setText("Whats this?");
        vbl->addWidget(tb);

        tb = new QToolButton(this);
        connect(tb,SIGNAL(clicked()),this,SLOT(aboutQt()));

        tb->setText("What is Qt?");
        vbl->addWidget(tb);
        vbl->addStretch(1);

        lay->addLayout(hbl,lay->rowCount()-1,1);
        hbl=0;
        vbl=0;
    }


    // add some test gifts
    {
        for(int i=0; i < 24; ++i)
            addGiftForDay(i+1,tr("1 (debug) kiss"));
    }
}


void AdventsKalender::addGiftForDay(const int day, const QString & gift)
{
    // remove test gift
    m_gifts.remove(day,tr("1 (debug) kiss").arg(day));
    // add gift
    m_gifts.insert(day,gift);
}

void AdventsKalender::showGiftForDay(const int day)
{
    const QDate today(QDate::currentDate());

    QStringList gifts(m_gifts.values(day));
    if (gifts.isEmpty()) gifts << tr("Nothing - poor you.");

    if (today.month()==11)
    {
        QMessageBox::information(this,tr("Still November ..."),tr("Only %1 days of waiting left.").arg(30-today.day()+1));
    }
    else if (today.month()==12)
    {
        if (today.day() > day)
            QMessageBox::information(this,tr("Day: %1").arg(day),tr("Your gift was:")+"\n\n"+gifts.join("\n"));
        else if (today.day() == day)
            QMessageBox::information(this,tr("Day: %1").arg(day),tr("Your gift is:")+"\n\n"+gifts.join("\n"));
        else if (today.day() < day)
            QMessageBox::information(this,tr("Day: %1").arg(day),tr("Stop snooping.\n\nSanta can see you!"));
    }
    else
    {
        QMessageBox::information(this,tr("Day: %1").arg(day),tr("In December your gift was:")+"\n\n"+gifts.join("\n"));
    }
}


void AdventsKalender::aboutThis()
{
    const QString msg =
            "This program was created in approximately 4 hours after reading"            "\n"
            "    http://www.qtforum.de/forum/viewtopic.php?t=13022"                      "\n"
            "                                                                             \n"
            "I use some free chrismasicons (not shipped with this code) in a subfolder"  "\n"
            "'./ress/day1.png' to './ress/day24.png' - see the shipped iconRess.qrc."    "\n"
            "                                                                             \n"
            "You can (and should) simply put images into this folder and name them the"  "\n"
            "same - best use images of squared size - they should be at least 48x48 up"  "\n"
            "to 256x256 pixels of size - less will be shown smaller, more are resized"   "\n"
            "to 48x48 pixels and will lead to artifacts and ugliness."                   "\n"
            "                                                                             \n"
            "If the qrc-alias returns null images, simple text will be shown."           "\n"
            "By default each day will give you one kiss, if you happen to add own"       "\n"
            "gifts they get removed."                                                    "\n"
            "                                                                             \n"
            "The programm will give you no gifts in November, in December you get only"  "\n"
            "gifts up to the current day - the other months you see what gifts you got." "\n"
            "                                                                             \n"
            "                                                                             \n"
            "Peace to all."
            ;

    QMessageBox::about(this,tr("About this application"),tr(msg.toAscii()));
}

void AdventsKalender::aboutQt()
{
    QMessageBox::aboutQt(this,tr("About Qt"));
}
